package javax.sound.midi;

import com.sun.media.sound.JDK13Services;
import java.io.OutputStream;
import java.util.Iterator;
import javax.sound.midi.spi.MidiFileWriter;
import java.util.HashSet;
import javax.sound.midi.spi.MidiFileReader;
import java.io.File;
import java.net.URL;
import java.io.IOException;
import javax.sound.midi.spi.SoundbankReader;
import java.io.InputStream;
import com.sun.media.sound.AutoConnectSequencer;
import com.sun.media.sound.MidiDeviceTransmitterEnvelope;
import com.sun.media.sound.MidiDeviceReceiverEnvelope;
import com.sun.media.sound.ReferenceCountingDevice;
import java.util.List;
import javax.sound.midi.spi.MidiDeviceProvider;
import java.util.ArrayList;

public class MidiSystem
{
    private MidiSystem() {
    }
    
    public static MidiDevice.Info[] getMidiDeviceInfo() {
        ArrayList<Object> list = new ArrayList();
        List<Object> midiDeviceProviders = getMidiDeviceProviders();
        for (int i = 0; i < midiDeviceProviders.size(); ++i) {
            MidiDevice.Info[] deviceInfo = ((MidiDeviceProvider)midiDeviceProviders.get(i)).getDeviceInfo();
            for (int j = 0; j < deviceInfo.length; ++j) {
                list.add(deviceInfo[j]);
            }
        }
        return (MidiDevice.Info[])(MidiDevice.Info[])list.toArray(new MidiDevice.Info[0]);
    }
    
    public static MidiDevice getMidiDevice(MidiDevice.Info info) throws MidiUnavailableException {
        List<Object> midiDeviceProviders = getMidiDeviceProviders();
        int i = 0;
        while (i < midiDeviceProviders.size()) {
            MidiDeviceProvider midiDeviceProvider = (MidiDeviceProvider)midiDeviceProviders.get(i);
            if (midiDeviceProvider.isDeviceSupported(info)) {
                return midiDeviceProvider.getDevice(info);
            }
            ++i;
        }
        throw new IllegalArgumentException("Requested device not installed: " + info);
    }
    
    public static Receiver getReceiver() throws MidiUnavailableException {
        MidiDevice defaultDeviceWrapper = getDefaultDeviceWrapper(Receiver.class);
        Object receiver;
        if ((defaultDeviceWrapper instanceof ReferenceCountingDevice)) {
            receiver = ((ReferenceCountingDevice)defaultDeviceWrapper).getReceiverReferenceCounting();
        }
        else {
            receiver = defaultDeviceWrapper.getReceiver();
        }
        if (!(receiver instanceof MidiDeviceReceiver)) {
            receiver = new MidiDeviceReceiverEnvelope(defaultDeviceWrapper, receiver);
            return receiver;
        }
        return receiver;
    }
    
    public static Transmitter getTransmitter() throws MidiUnavailableException {
        MidiDevice defaultDeviceWrapper = getDefaultDeviceWrapper(Transmitter.class);
        Object transmitter;
        if ((defaultDeviceWrapper instanceof ReferenceCountingDevice)) {
            transmitter = ((ReferenceCountingDevice)defaultDeviceWrapper).getTransmitterReferenceCounting();
        }
        else {
            transmitter = defaultDeviceWrapper.getTransmitter();
        }
        if (!(transmitter instanceof MidiDeviceTransmitter)) {
            transmitter = new MidiDeviceTransmitterEnvelope(defaultDeviceWrapper, transmitter);
            return transmitter;
        }
        return transmitter;
    }
    
    public static Synthesizer getSynthesizer() throws MidiUnavailableException {
        return (Synthesizer)getDefaultDeviceWrapper(Synthesizer.class);
    }
    
    public static Sequencer getSequencer() throws MidiUnavailableException {
        return getSequencer(true);
    }
    
    public static Sequencer getSequencer(boolean b) throws MidiUnavailableException {
        Sequencer sequencer = (MidiDevice)getDefaultDeviceWrapper(Sequencer.class);
        if (b) {
            Receiver receiver = null;
            MidiUnavailableException ex = null;
            try {
                Synthesizer synthesizer = getSynthesizer();
                if ((synthesizer instanceof ReferenceCountingDevice)) {
                    receiver = ((ReferenceCountingDevice)synthesizer).getReceiverReferenceCounting();
                }
                else {
                    synthesizer.open();
                    try {
                        receiver = synthesizer.getReceiver();
                    }
                    finally {
                        if (receiver == null) {
                            synthesizer.close();
                        }
                    }
                }
            }
            catch (MidiUnavailableException ex2) {
                if ((ex2 instanceof MidiUnavailableException)) {
                    ex = ex2;
                }
            }
            if (receiver == null) {
                try {
                    receiver = getReceiver();
                }
                catch (Exception ex3) {
                    if ((ex3 instanceof MidiUnavailableException)) {
                        ex = (MidiUnavailableException)ex3;
                    }
                }
            }
            if (receiver != null) {
                sequencer.getTransmitter().setReceiver(receiver);
                if ((sequencer instanceof AutoConnectSequencer)) {
                    ((AutoConnectSequencer)sequencer).setAutoConnect(receiver);
                }
            }
            else {
                if (ex != null) {
                    throw ex;
                }
                throw new MidiUnavailableException("no receiver available");
            }
        }
        return sequencer;
    }
    
    public static Soundbank getSoundbank(InputStream inputStream) throws InvalidMidiDataException, IOException {
        List<Object> soundbankReaders = getSoundbankReaders();
        int i = 0;
        while (i < soundbankReaders.size()) {
            Soundbank soundbank = ((SoundbankReader)soundbankReaders.get(i)).getSoundbank(inputStream);
            if (soundbank != null) {
                return soundbank;
            }
            ++i;
        }
        throw new InvalidMidiDataException("cannot get soundbank from stream");
    }
    
    public static Soundbank getSoundbank(URL url) throws InvalidMidiDataException, IOException {
        List<Object> soundbankReaders = getSoundbankReaders();
        int i = 0;
        while (i < soundbankReaders.size()) {
            Soundbank soundbank = ((SoundbankReader)soundbankReaders.get(i)).getSoundbank(url);
            if (soundbank != null) {
                return soundbank;
            }
            ++i;
        }
        throw new InvalidMidiDataException("cannot get soundbank from stream");
    }
    
    public static Soundbank getSoundbank(File file) throws InvalidMidiDataException, IOException {
        List<Object> soundbankReaders = getSoundbankReaders();
        int i = 0;
        while (i < soundbankReaders.size()) {
            Soundbank soundbank = ((SoundbankReader)soundbankReaders.get(i)).getSoundbank(file);
            if (soundbank != null) {
                return soundbank;
            }
            ++i;
        }
        throw new InvalidMidiDataException("cannot get soundbank from stream");
    }
    
    public static MidiFileFormat getMidiFileFormat(InputStream inputStream) throws InvalidMidiDataException, IOException {
        List<Object> midiFileReaders = getMidiFileReaders();
        MidiFileFormat midiFileFormat = null;
        int i = 0;
        while (i < midiFileReaders.size()) {
            MidiFileReader midiFileReader = (MidiFileReader)midiFileReaders.get(i);
            try {
                midiFileFormat = midiFileReader.getMidiFileFormat(inputStream);
                break;
            }
            catch (InvalidMidiDataException ex) {
                ++i;
            }
        }
        if (midiFileFormat == null) {
            throw new InvalidMidiDataException("input stream is not a supported file type");
        }
        return midiFileFormat;
    }
    
    public static MidiFileFormat getMidiFileFormat(URL url) throws InvalidMidiDataException, IOException {
        List<Object> midiFileReaders = getMidiFileReaders();
        MidiFileFormat midiFileFormat = null;
        int i = 0;
        while (i < midiFileReaders.size()) {
            MidiFileReader midiFileReader = (MidiFileReader)midiFileReaders.get(i);
            try {
                midiFileFormat = midiFileReader.getMidiFileFormat(url);
                break;
            }
            catch (InvalidMidiDataException ex) {
                ++i;
            }
        }
        if (midiFileFormat == null) {
            throw new InvalidMidiDataException("url is not a supported file type");
        }
        return midiFileFormat;
    }
    
    public static MidiFileFormat getMidiFileFormat(File file) throws InvalidMidiDataException, IOException {
        List<Object> midiFileReaders = getMidiFileReaders();
        MidiFileFormat midiFileFormat = null;
        int i = 0;
        while (i < midiFileReaders.size()) {
            MidiFileReader midiFileReader = (MidiFileReader)midiFileReaders.get(i);
            try {
                midiFileFormat = midiFileReader.getMidiFileFormat(file);
                break;
            }
            catch (InvalidMidiDataException ex) {
                ++i;
            }
        }
        if (midiFileFormat == null) {
            throw new InvalidMidiDataException("file is not a supported file type");
        }
        return midiFileFormat;
    }
    
    public static Sequence getSequence(InputStream inputStream) throws InvalidMidiDataException, IOException {
        List<Object> midiFileReaders = getMidiFileReaders();
        Sequence sequence = null;
        int i = 0;
        while (i < midiFileReaders.size()) {
            MidiFileReader midiFileReader = (MidiFileReader)midiFileReaders.get(i);
            try {
                sequence = midiFileReader.getSequence(inputStream);
                break;
            }
            catch (InvalidMidiDataException ex) {
                ++i;
            }
        }
        if (sequence == null) {
            throw new InvalidMidiDataException("could not get sequence from input stream");
        }
        return sequence;
    }
    
    public static Sequence getSequence(URL url) throws InvalidMidiDataException, IOException {
        List<Object> midiFileReaders = getMidiFileReaders();
        Sequence sequence = null;
        int i = 0;
        while (i < midiFileReaders.size()) {
            MidiFileReader midiFileReader = (MidiFileReader)midiFileReaders.get(i);
            try {
                sequence = midiFileReader.getSequence(url);
                break;
            }
            catch (InvalidMidiDataException ex) {
                ++i;
            }
        }
        if (sequence == null) {
            throw new InvalidMidiDataException("could not get sequence from URL");
        }
        return sequence;
    }
    
    public static Sequence getSequence(File file) throws InvalidMidiDataException, IOException {
        List<Object> midiFileReaders = getMidiFileReaders();
        Sequence sequence = null;
        int i = 0;
        while (i < midiFileReaders.size()) {
            MidiFileReader midiFileReader = (MidiFileReader)midiFileReaders.get(i);
            try {
                sequence = midiFileReader.getSequence(file);
                break;
            }
            catch (InvalidMidiDataException ex) {
                ++i;
            }
        }
        if (sequence == null) {
            throw new InvalidMidiDataException("could not get sequence from file");
        }
        return sequence;
    }
    
    public static int[] getMidiFileTypes() {
        List<Object> midiFileWriters = getMidiFileWriters();
        HashSet<Integer> set = new HashSet();
        for (int i = 0; i < midiFileWriters.size(); ++i) {
            int[] midiFileTypes = ((MidiFileWriter)midiFileWriters.get(i)).getMidiFileTypes();
            for (int j = 0; j < midiFileTypes.length; ++j) {
                set.add(new Integer(midiFileTypes[j]));
            }
        }
        int[] array = new int[set.size()];
        int n = 0;
        Iterator<Object> iterator = set.iterator();
        while (iterator.hasNext()) {
            array[n++] = (int)iterator.next();
        }
        return array;
    }
    
    public static boolean isFileTypeSupported(int fileType) {
        List<Object> midiFileWriters = getMidiFileWriters();
        int i = 0;
        while (i < midiFileWriters.size()) {
            if (((MidiFileWriter)midiFileWriters.get(i)).isFileTypeSupported(fileType)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static int[] getMidiFileTypes(Sequence sequence) {
        List<Object> midiFileWriters = getMidiFileWriters();
        HashSet<Integer> set = new HashSet();
        for (int i = 0; i < midiFileWriters.size(); ++i) {
            int[] midiFileTypes = ((MidiFileWriter)midiFileWriters.get(i)).getMidiFileTypes(sequence);
            for (int j = 0; j < midiFileTypes.length; ++j) {
                set.add(new Integer(midiFileTypes[j]));
            }
        }
        int[] array = new int[set.size()];
        int n = 0;
        Iterator<Object> iterator = set.iterator();
        while (iterator.hasNext()) {
            array[n++] = (int)iterator.next();
        }
        return array;
    }
    
    public static boolean isFileTypeSupported(int fileType, Sequence sequence) {
        List<Object> midiFileWriters = getMidiFileWriters();
        int i = 0;
        while (i < midiFileWriters.size()) {
            if (((MidiFileWriter)midiFileWriters.get(i)).isFileTypeSupported(fileType, sequence)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static int write(Sequence sequence, int fileType, OutputStream outputStream) throws IOException {
        List<Object> midiFileWriters = getMidiFileWriters();
        int write = -2;
        int i = 0;
        while (i < midiFileWriters.size()) {
            MidiFileWriter midiFileWriter = (MidiFileWriter)midiFileWriters.get(i);
            if (midiFileWriter.isFileTypeSupported(fileType, sequence)) {
                write = midiFileWriter.write(sequence, fileType, outputStream);
                break;
            }
            ++i;
        }
        if (write == -2) {
            throw new IllegalArgumentException("MIDI file type is not supported");
        }
        return write;
    }
    
    public static int write(Sequence sequence, int fileType, File file) throws IOException {
        List<Object> midiFileWriters = getMidiFileWriters();
        int write = -2;
        int i = 0;
        while (i < midiFileWriters.size()) {
            MidiFileWriter midiFileWriter = (MidiFileWriter)midiFileWriters.get(i);
            if (midiFileWriter.isFileTypeSupported(fileType, sequence)) {
                write = midiFileWriter.write(sequence, fileType, file);
                break;
            }
            ++i;
        }
        if (write == -2) {
            throw new IllegalArgumentException("MIDI file type is not supported");
        }
        return write;
    }
    
    private static List getMidiDeviceProviders() {
        return getProviders(MidiDeviceProvider.class);
    }
    
    private static List getSoundbankReaders() {
        return getProviders(SoundbankReader.class);
    }
    
    private static List getMidiFileWriters() {
        return getProviders(MidiFileWriter.class);
    }
    
    private static List getMidiFileReaders() {
        return getProviders(MidiFileReader.class);
    }
    
    private static MidiDevice getDefaultDeviceWrapper(Class clazz) throws MidiUnavailableException {
        try {
            return getDefaultDevice(clazz);
        }
        catch (IllegalArgumentException cause) {
            MidiUnavailableException ex = new MidiUnavailableException();
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private static MidiDevice getDefaultDevice(Class clazz) {
        List<Object> midiDeviceProviders = getMidiDeviceProviders();
        String defaultProviderClassName = JDK13Services.getDefaultProviderClassName(clazz);
        String defaultInstanceName = JDK13Services.getDefaultInstanceName(clazz);
        if (defaultProviderClassName != null) {
            MidiDeviceProvider namedProvider = getNamedProvider(defaultProviderClassName, midiDeviceProviders);
            if (namedProvider != null) {
                if (defaultInstanceName != null) {
                    MidiDevice namedDevice = getNamedDevice(defaultInstanceName, namedProvider, clazz);
                    if (namedDevice != null) {
                        return namedDevice;
                    }
                }
                MidiDevice firstDevice = getFirstDevice(namedProvider, clazz);
                if (firstDevice != null) {
                    return firstDevice;
                }
            }
        }
        if (defaultInstanceName != null) {
            MidiDevice namedDevice2 = getNamedDevice(defaultInstanceName, midiDeviceProviders, clazz);
            if (namedDevice2 != null) {
                return namedDevice2;
            }
        }
        MidiDevice firstDevice2 = getFirstDevice(midiDeviceProviders, clazz);
        if (firstDevice2 != null) {
            return firstDevice2;
        }
        throw new IllegalArgumentException("Requested device not installed");
    }
    
    private static MidiDeviceProvider getNamedProvider(String anObject, List list) {
        int i = 0;
        while (i < list.size()) {
            MidiDeviceProvider midiDeviceProvider = (MidiDeviceProvider)list.get(i);
            if (midiDeviceProvider.getClass().getName().equals(anObject)) {
                return midiDeviceProvider;
            }
            ++i;
        }
        return null;
    }
    
    private static MidiDevice getNamedDevice(String s, MidiDeviceProvider midiDeviceProvider, Class clazz) {
        MidiDevice namedDevice = getNamedDevice(s, midiDeviceProvider, clazz, false, false);
        if (namedDevice != null) {
            return namedDevice;
        }
        if (clazz == Receiver.class) {
            MidiDevice namedDevice2 = getNamedDevice(s, midiDeviceProvider, clazz, true, false);
            if (namedDevice2 != null) {
                return namedDevice2;
            }
        }
        return null;
    }
    
    private static MidiDevice getNamedDevice(String anObject, MidiDeviceProvider midiDeviceProvider, Class clazz, boolean b, boolean b2) {
        MidiDevice.Info[] deviceInfo = midiDeviceProvider.getDeviceInfo();
        for (int i = 0; i < deviceInfo.length; ++i) {
            if (deviceInfo[i].getName().equals(anObject)) {
                MidiDevice device = midiDeviceProvider.getDevice(deviceInfo[i]);
                if (isAppropriateDevice(device, clazz, b, b2)) {
                    return device;
                }
            }
        }
        return null;
    }
    
    private static MidiDevice getNamedDevice(String s, List list, Class clazz) {
        MidiDevice namedDevice = getNamedDevice(s, list, clazz, false, false);
        if (namedDevice != null) {
            return namedDevice;
        }
        if (clazz == Receiver.class) {
            MidiDevice namedDevice2 = getNamedDevice(s, list, clazz, true, false);
            if (namedDevice2 != null) {
                return namedDevice2;
            }
        }
        return null;
    }
    
    private static MidiDevice getNamedDevice(String s, List list, Class clazz, boolean b, boolean b2) {
        int i = 0;
        while (i < list.size()) {
            MidiDevice namedDevice = getNamedDevice(s, (MidiDeviceProvider)list.get(i), clazz, b, b2);
            if (namedDevice != null) {
                return namedDevice;
            }
            ++i;
        }
        return null;
    }
    
    private static MidiDevice getFirstDevice(MidiDeviceProvider midiDeviceProvider, Class clazz) {
        MidiDevice firstDevice = getFirstDevice(midiDeviceProvider, clazz, false, false);
        if (firstDevice != null) {
            return firstDevice;
        }
        if (clazz == Receiver.class) {
            MidiDevice firstDevice2 = getFirstDevice(midiDeviceProvider, clazz, true, false);
            if (firstDevice2 != null) {
                return firstDevice2;
            }
        }
        return null;
    }
    
    private static MidiDevice getFirstDevice(MidiDeviceProvider midiDeviceProvider, Class clazz, boolean b, boolean b2) {
        MidiDevice.Info[] deviceInfo = midiDeviceProvider.getDeviceInfo();
        int i = 0;
        while (i < deviceInfo.length) {
            MidiDevice device = midiDeviceProvider.getDevice(deviceInfo[i]);
            if (isAppropriateDevice(device, clazz, b, b2)) {
                return device;
            }
            ++i;
        }
        return null;
    }
    
    private static MidiDevice getFirstDevice(List list, Class clazz) {
        MidiDevice firstDevice = getFirstDevice(list, clazz, false, false);
        if (firstDevice != null) {
            return firstDevice;
        }
        if (clazz == Receiver.class) {
            MidiDevice firstDevice2 = getFirstDevice(list, clazz, true, false);
            if (firstDevice2 != null) {
                return firstDevice2;
            }
        }
        return null;
    }
    
    private static MidiDevice getFirstDevice(List list, Class clazz, boolean b, boolean b2) {
        int i = 0;
        while (i < list.size()) {
            MidiDevice firstDevice = getFirstDevice((MidiDeviceProvider)list.get(i), clazz, b, b2);
            if (firstDevice != null) {
                return firstDevice;
            }
            ++i;
        }
        return null;
    }
    
    private static boolean isAppropriateDevice(MidiDevice midiDevice, Class clazz, boolean b, boolean b2) {
        return clazz.isInstance(midiDevice) || (((!(midiDevice instanceof Sequencer) && !(midiDevice instanceof Synthesizer)) || (midiDevice instanceof Sequencer && b2) || (midiDevice instanceof Synthesizer && b)) && ((clazz == Receiver.class && midiDevice.getMaxReceivers() != 0) || (clazz == Transmitter.class && midiDevice.getMaxTransmitters() != 0)));
    }
    
    private static List getProviders(Class serviceClass) {
        return JDK13Services.getProviders(serviceClass);
    }
}
